@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: __init__.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: __init__.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Library initialization script
 #
@@ -26,7 +26,6 @@
 import crack
 import util
 import config
-import gnomemisc
 import datahandler
 import io
 import entry
