@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: fpm.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: fpm.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for handling Figaro's Password Manager data
 #
@@ -168,7 +168,7 @@ class FPM(base.DataHandler):
 			e = entrystore.get_entry(iter)
 
 			if type(e) != entry.FolderEntry:
-				e = entry.convert_entry_generic(e)
+				e = e.convert_generic()
 
 				xml += "		<PasswordItem>\n"
 				xml += "			<title>%s</title>\n" % e.name
