@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: gpass.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: gpass.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for handling GPass data
 #
@@ -87,7 +87,7 @@ def encrypt(plaintext, password):
 class GPass04(base.DataHandler):
 	"Data handler for GPass 0.4.x data"
 
-	name		= "GPass 0.4.x (or older)"
+	name		= "GPass 0.4.x"
 	importer	= True
 	exporter	= True
 	encryption	= True
@@ -108,7 +108,7 @@ class GPass04(base.DataHandler):
 			e = entrystore.get_entry(iter)
 
 			if type(e) != entry.FolderEntry:
-				e = entry.convert_entry_generic(e)
+				e = e.convert_generic()
 
 				data += e.name + "\n"
 				data += e[entry.UsernameField] + "\n"
@@ -314,7 +314,7 @@ class GPass05(base.DataHandler):
 				foldermap[path] = id
 
 			elif type(e) != entry.GenericEntry:
-				e = entry.convert_entry_generic(e)
+				e = e.convert_generic()
 
 
 			entrydata	= ""
