@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: netrc.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: netrc.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for handling .netrc files
 #
@@ -78,7 +78,7 @@ class NetRC(base.DataHandler):
 		# set up a lexical parser
 		datafp = StringIO.StringIO(netrc)
 		lexer = shlex.shlex(datafp)
-		lexer.wordchars += r"""!"#$%&'()*+,-./:;<=>?@[\]^_`{|}~"""
+		lexer.wordchars += r"!\"#$%&'()*+,-./:;<=>?@[\]^_`{|}~"
 
 		while 1:
 			# look for a machine, default or macdef top-level keyword
@@ -89,7 +89,7 @@ class NetRC(base.DataHandler):
 
 			elif tt == "machine":
 				name = lexer.get_token()
-	
+
 			elif tt == "default":
 				name = "default"
 
