@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: pwsafe.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: pwsafe.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for handling PasswordSafe data
 #
@@ -405,7 +405,7 @@ class PasswordSafe1(base.DataHandler):
 			e = entrystore.get_entry(iter)
 
 			if type(e) != entry.FolderEntry:
-				e = entry.convert_entry_generic(e)
+				e = e.convert_generic()
 
 				edata = ""
 				edata += create_field(e.name.encode(enc, "replace") + "\xAD" + e[entry.UsernameField].encode("iso-8859-1"))
@@ -563,7 +563,7 @@ class PasswordSafe2(base.DataHandler):
 			e = entrystore.get_entry(iter)
 
 			if type(e) != entry.FolderEntry:
-				e = entry.convert_entry_generic(e)
+				e = e.convert_generic()
 
 				uuid = util.random_string(16)
 
