@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: rvl.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: rvl.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for handling Revelation data
 #
@@ -25,8 +25,9 @@
 
 import base
 from revelation import config, data, entry, util
+from revelation.bundle import luks
 
-import re, xml.dom.minidom, zlib
+import re, StringIO, struct, xml.dom.minidom, zlib
 
 from xml.parsers.expat import ExpatError
 from Crypto.Cipher import AES
@@ -50,7 +51,7 @@ class RevelationXML(base.DataHandler):
 		"Looks up an entry type based on an identifier"
 
 		for entrytype in entry.ENTRYLIST:
-			if entrytype.id == typename:
+			if entrytype().id == typename:
 				return entrytype
 
 		else:
@@ -195,11 +196,6 @@ class RevelationXML(base.DataHandler):
 	def import_data(self, input, password = None):
 		"Imports data from a data stream to an entrystore"
 
-		# Workaround for incorrect encoding in files generated by version 0.4.0.
-		# To be removed in version 0.5.0.
-		if input[:44] == '<?xml version="1.0" encoding="iso-8859-1" ?>':
-			input = '<?xml version="1.0" encoding="utf-8" ?>' + input[44:]
-
 		RevelationXML.check(self, input)
 
 		try:
@@ -387,3 +383,127 @@ class Revelation(RevelationXML):
 
 		return entrystore
 
+
+class RevelationLUKS(RevelationXML):
+	"Handler for Revelation XML using the LUKS on disk format"
+
+	name 		= "Revelation LUKS"
+	importer	= True
+	exporter	= True
+	encryption	= True
+
+	def __init__(self):
+		RevelationXML.__init__(self)
+		self.luks_header = None
+		self.luks_buff = None
+		self.current_slot = False
+
+
+	def check(self, input):
+		"Checks if the data is valid"
+
+		if input is None:
+			raise base.FormatError
+
+		sbuf = StringIO.StringIO(input)
+
+		l = luks.LuksFile()
+
+		try:
+			l.load_from_file(sbuf)
+
+		except:
+			l.close()
+			raise base.FormatError
+
+		l.close()
+
+
+	def detect(self, input):
+		"Checks if the handler can guarantee to use the data"
+
+		try:
+			self.check(input)
+			return True
+
+		except ( base.FormatError, base.VersionError ):
+			return False
+
+
+	def export_data(self, entrystore, password):
+		"Exports data from an entrystore"
+
+		# check and pad password
+		if password is None:
+			raise base.PasswordError
+
+		# generate and compress XML
+		data = RevelationXML.export_data(self, entrystore)
+		data = zlib.compress(data)
+
+		# data needs to be padded to 512 bytes
+		# We use Merkle-Damgard length padding (1 bit followed by 0 bits + size)
+		# http://en.wikipedia.org/wiki/Merkle-Damg%C3%A5rd_hash_function
+		padlen = 512 - (len(data) % 512)
+
+		if padlen < 4:
+			padlen = 512 + padlen
+
+		if padlen > 4:
+			data += "\x80" + "\x00" * (padlen - 5)
+
+		data += struct.pack("<I", padlen)
+
+		# create a new luks file in memory
+		buffer		= StringIO.StringIO()
+		luksfile	= luks.LuksFile()
+		luksfile.create(buffer, "aes", "cbc-essiv:sha256", "sha1", 16, 400)
+
+		luksfile.set_key(0, password, 5000, 400)
+
+		# encrypt the data
+		luksfile.encrypt_data(0, data)
+		buffer.seek(0)
+
+		return buffer.read()
+
+
+	def import_data(self, input, password):
+		"Imports data into an entrystore"
+
+		# check password
+		if password is None:
+			raise base.PasswordError
+
+		# create a LuksFile
+		buffer		= StringIO.StringIO(input)
+		luksfile	= luks.LuksFile()
+
+		try:
+			luksfile.load_from_file(buffer)
+
+		except:
+			luksfile.close()
+			buffer.close()
+			raise base.FormatError
+
+		slot = luksfile.open_any_key(password)
+
+		if slot == None:
+			luksfile.close()
+			buffer.close()
+			raise base.PasswordError
+
+		data = luksfile.decrypt_data(0, luksfile.data_length())
+
+		# remove the pad, and decompress
+		padlen = struct.unpack("<I", data[-4:])[0]
+		data = zlib.decompress(data[0:-padlen])
+
+		if data.strip()[:5] != "<?xml":
+			raise base.FormatError
+
+		entrystore = RevelationXML.import_data(self, data)
+
+		return entrystore
+
