@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: xhtml.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: xhtml.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for exporting to XHTML files
 #
@@ -26,7 +26,9 @@
 import base
 from revelation import config, data, entry
 
-import time
+import gettext, time
+
+_ = gettext.gettext
 
 
 IMAGEPATH	= "http://oss.codepoet.no/revelation/img/fileicons"
@@ -415,7 +417,7 @@ class XHTML(base.DataHandler):
 		xhtml += "</head>\n"
 		xhtml += "\n"
 		xhtml += "<body>\n"
-		xhtml += "<h1>Revelation account list</h1>\n"
+		xhtml += "<h1>" + _('Revelation account list') + "</h1>\n"
 		xhtml += "\n"
 
 		return xhtml
@@ -456,14 +458,14 @@ class XHTML(base.DataHandler):
 
 		# generate the xhtml
 		xhtml = ""
-		xhtml += "	<h2><img src=\"%s/sidebar/accountlist.png\" class=\"icon\" alt=\"Account list\" />Account list</h2>\n" % IMAGEPATH
+		xhtml += "	<h2><img src=\"%s/sidebar/accountlist.png\" class=\"icon\" alt=\"Account list\" />" + _('Account list') + "</h2>\n" % IMAGEPATH
 		xhtml += "\n"
 
 		for entrytype in entry.ENTRYLIST:
 			if not entries.has_key(entrytype):
 				continue
 
-			xhtml += "	<h3>%s:</h3>\n" % entrytype.typename
+			xhtml += "	<h3>%s:</h3>\n" % entrytype().typename
 			xhtml += "\n"
 			xhtml += "	<ul class=\"accountlist\">\n"
 
@@ -485,10 +487,10 @@ class XHTML(base.DataHandler):
 		xhtml = ""
 		xhtml += "	<h2 style=\"margin-top: 0px;\"><img src=\"%s/sidebar/file.png\" class=\"icon\" alt=\"File info\" />File info</h2>\n" % IMAGEPATH
 		xhtml += "\n"
-		xhtml += "	<h3>Created:</h3>\n"
+		xhtml += "	<h3>" + _('Created:') + "</h3>\n"
 		xhtml += "	<p>%s</p>\n" % time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
 		xhtml += "\n"
-		xhtml += "	<h3>Revelation version:</h3>\n"
+		xhtml += "	<h3>" + _('Revelation version:') + "</h3>\n"
 		xhtml += "	<p>%s</p>\n" % config.VERSION
 		xhtml += "\n"
 
