@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: dialog.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: dialog.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module with dialogs
 #
@@ -25,14 +25,24 @@
 
 import config, datahandler, entry, io, ui, util
 
-import gobject, gtk, gnome.ui, urllib
+import gettext, gnome.ui, gobject, gtk, urllib
 
+_ = gettext.gettext
 
 
 EVENT_FILTER		= None
 UNIQUE_DIALOGS		= {}
 
 
+gtk.rc_parse_string("""
+	style "hig" {
+		GtkDialog::content-area-border	= 0
+		GtkDialog::action-area-border	= 0
+	}
+
+	class "GtkDialog" style "hig"
+""")
+
 
 ##### EXCEPTIONS #####
 
@@ -53,17 +63,18 @@ class Dialog(gtk.Dialog):
 			gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR
 		)
 
-		self.set_border_width(6)
+		self.set_border_width(12)
 		self.vbox.set_spacing(12)
 		self.set_resizable(False)
 		self.set_modal(True)
 
 		self.connect("key_press_event", self.__cb_keypress)
+		self.connect("realize", self.__cb_realize)
 
 		for stock, response in buttons:
 			self.add_button(stock, response)
 
-		if default is not None:
+		if default != None:
 			self.set_default_response(default)
 
 		elif len(buttons) > 0:
@@ -78,6 +89,12 @@ class Dialog(gtk.Dialog):
 			self.response(gtk.RESPONSE_CLOSE)
 
 
+	def __cb_realize(self, widget, data = None):
+		"Callback for widget realization"
+
+		self.action_area.set_spacing(6)
+
+
 	def get_button(self, index):
 		"Get one of the dialog buttons"
 
@@ -197,11 +214,11 @@ class Message(Dialog):
 		# hbox with image and contents
 		hbox = ui.HBox()
 		hbox.set_spacing(12)
-		hbox.set_border_width(6)
 		self.vbox.pack_start(hbox)
+		self.vbox.set_spacing(24)
 
 		# set up image
-		if stockimage is not None:
+		if stockimage != None:
 			image = ui.Image(stockimage, gtk.ICON_SIZE_DIALOG)
 			image.set_alignment(0.5, 0)
 			hbox.pack_start(image, False, False)
@@ -211,10 +228,7 @@ class Message(Dialog):
 		self.contents.set_spacing(10)
 		hbox.pack_start(self.contents)
 
-		label = ui.Label(
-			"<span size=\"larger\" weight=\"bold\">%s</span>\n\n%s" %
-			( util.escape_markup(title), text)
-		)
+		label = ui.Label("<span size=\"larger\" weight=\"bold\">%s</span>\n\n%s" % ( util.escape_markup(title), text))
 		label.set_alignment(0, 0)
 		label.set_selectable(True)
 		self.contents.pack_start(label)
@@ -259,7 +273,7 @@ class Warning(Message):
 	"Displays a warning message"
 
 	def __init__(self, parent, title, text, buttons = ( ( gtk.STOCK_OK, gtk.RESPONSE_OK ), ), default = None):
-		Message.__init__(self, parent, title, text, gtk.STOCK_DIALOG_WARNING, buttons, default)
+		Message.__init__(self, parent, title, text, ui.STOCK_WARNING, buttons, default)
 
 
 
@@ -270,7 +284,7 @@ class FileChanged(Warning):
 
 	def __init__(self, parent, filename):
 		Warning.__init__(
-			self, parent, "File has changed", "The current file '%s' has changed. Do you want to reload it?" % filename,
+			self, parent, _('File has changed'), _('The current file \'%s\' has changed. Do you want to reload it?') % filename,
 			( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( ui.STOCK_RELOAD, gtk.RESPONSE_OK ) )
 		)
 
@@ -316,8 +330,8 @@ class FileChangesNew(FileChanges):
 
 	def __init__(self, parent):
 		FileChanges.__init__(
-			self, parent, "Save changes to current file?",
-			"You have made changes which have not been saved. If you create a new file without saving then these changes will be lost."
+			self, parent, _('Save changes to current file?'),
+			_('You have made changes which have not been saved. If you create a new file without saving then these changes will be lost.')
 		)
 
 
@@ -327,8 +341,8 @@ class FileChangesOpen(FileChanges):
 
 	def __init__(self, parent):
 		FileChanges.__init__(
-			self, parent, "Save changes before opening?",
-			"You have made changes which have not been saved. If you open a different file without saving then these changes will be lost."
+			self, parent, _('Save changes before opening?'),
+			_('You have made changes which have not been saved. If you open a different file without saving then these changes will be lost.')
 		)
 
 
@@ -338,20 +352,20 @@ class FileChangesQuit(FileChanges):
 
 	def __init__(self, parent):
 		FileChanges.__init__(
-			self, parent, "Save changes before quitting?",
-			"You have made changes which have not been saved. If you quit without saving, then these changes will be lost."
+			self, parent, _('Save changes before quitting?'),
+			_('You have made changes which have not been saved. If you quit without saving, then these changes will be lost.')
 		)
 
 
 
-class FileOverwrite(Warning):
-	"Asks for confirmation when overwriting a file"
+class FileReplace(Warning):
+	"Asks for confirmation when replacing a file"
 
 	def __init__(self, parent, file):
 		Warning.__init__(
-			self, parent, "Overwrite existing file?",
-			"The file '%s' already exists - do you wish to replace this file?" % file,
-			( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( ui.STOCK_OVERWRITE, gtk.RESPONSE_OK ) ),
+			self, parent, _('Replace existing file?'),
+			_('The file \'%s\' already exists - do you wish to replace this file?') % file,
+			( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( ui.STOCK_REPLACE, gtk.RESPONSE_OK ) ),
 			gtk.RESPONSE_CANCEL
 		)
 
@@ -372,8 +386,8 @@ class FileSaveInsecure(Warning):
 
 	def __init__(self, parent):
 		Warning.__init__(
-			self, parent, "Save to insecure file?",
-			"You have chosen to save your passwords to an insecure (unencrypted) file format - if anyone has access to this file, they will be able to see your passwords.",
+			self, parent, _('Save to insecure file?'),
+			_('You have chosen to save your passwords to an insecure (unencrypted) file format - if anyone has access to this file, they will be able to see your passwords.'),
 			( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( gtk.STOCK_SAVE, gtk.RESPONSE_OK ) ), 0
 		)
 
@@ -461,13 +475,13 @@ class ExportFileSelector(FileSelector):
 
 	def __init__(self, parent):
 		FileSelector.__init__(
-			self, parent, "Select File to Export to",
+			self, parent, _('Select File to Export to'),
 			gtk.FILE_CHOOSER_ACTION_SAVE, ui.STOCK_EXPORT
 		)
 
 		# set up filetype dropdown
 		self.dropdown = ui.DropDown()
-		self.add_widget("Filetype", self.dropdown)
+		self.add_widget(_('Filetype'), self.dropdown)
 
 		for handler in datahandler.get_export_handlers():
 			self.dropdown.append_item(handler.name, None, handler)
@@ -501,15 +515,15 @@ class ImportFileSelector(FileSelector):
 
 	def __init__(self, parent):
 		FileSelector.__init__(
-			self, parent, "Select File to Import",
+			self, parent, _('Select File to Import'),
 			gtk.FILE_CHOOSER_ACTION_OPEN, ui.STOCK_IMPORT
 		)
 
 		# set up filetype dropdown
 		self.dropdown = ui.DropDown()
-		self.add_widget("Filetype", self.dropdown)
+		self.add_widget(_('Filetype'), self.dropdown)
 
-		self.dropdown.append_item("Automatically detect")
+		self.dropdown.append_item(_('Automatically detect'))
 
 		for handler in datahandler.get_import_handlers():
 			self.dropdown.append_item(handler.name, None, handler)
@@ -539,10 +553,20 @@ class OpenFileSelector(FileSelector):
 
 	def __init__(self, parent):
 		FileSelector.__init__(
-			self, parent, "Select File to Open",
+			self, parent, _('Select File to Open'),
 			gtk.FILE_CHOOSER_ACTION_OPEN, gtk.STOCK_OPEN
 		)
 
+		filter = gtk.FileFilter()
+		filter.set_name(_('Revelation files'))
+		filter.add_mime_type("application/x-revelation")
+		self.add_filter(filter)
+
+		filter = gtk.FileFilter()
+		filter.set_name(_('All files'))
+		filter.add_pattern("*")
+		self.add_filter(filter)
+
 
 
 class SaveFileSelector(FileSelector):
@@ -550,10 +574,36 @@ class SaveFileSelector(FileSelector):
 
 	def __init__(self, parent):
 		FileSelector.__init__(
-			self, parent, "Select File to Save to",
+			self, parent, _('Select File to Save to'),
 			gtk.FILE_CHOOSER_ACTION_SAVE, gtk.STOCK_SAVE
 		)
 
+		filter = gtk.FileFilter()
+		filter.set_name(_('Revelation files'))
+		filter.add_mime_type("application/x-revelation")
+		self.add_filter(filter)
+
+		filter = gtk.FileFilter()
+		filter.set_name(_('All files'))
+		filter.add_pattern("*")
+		self.add_filter(filter)
+
+		self.set_do_overwrite_confirmation(True)
+		self.connect("confirm-overwrite", self.__cb_confirm_overwrite)
+
+
+	def __cb_confirm_overwrite(self, widget, data = None):
+		"Handles confirm-overwrite signals"
+
+		try:
+			FileReplace(self, io.file_normpath(self.get_uri())).run()
+
+		except CancelError:
+			return gtk.FILE_CHOOSER_CONFIRMATION_SELECT_AGAIN
+
+		else:
+			return gtk.FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME
+
 
 
 ##### PASSWORD DIALOGS #####
@@ -603,18 +653,18 @@ class PasswordChange(Password):
 
 	def __init__(self, parent, password = None):
 		Password.__init__(
-			self, parent, "Enter new password",
-			"Enter a new password for the current data file. The file must be saved before the new password is applied.",
+			self, parent, _('Enter new password'),
+			_('Enter a new password for the current data file. The file must be saved before the new password is applied.'),
 			ui.STOCK_PASSWORD_CHANGE
 		)
 
 		self.password = password
 
 		if password is not None:
-			self.entry_current = self.add_entry("Current password")
+			self.entry_current = self.add_entry(_('Current password'))
 
-		self.entry_new		= self.add_entry("New password", ui.PasswordEntry())
-		self.entry_confirm	= self.add_entry("Confirm password")
+		self.entry_new		= self.add_entry(_('New password'), ui.PasswordEntry())
+		self.entry_confirm	= self.add_entry(_('Confirm password'))
 		self.entry_confirm.autocheck = False
 
 
@@ -627,10 +677,10 @@ class PasswordChange(Password):
 				raise CancelError
 
 			elif self.password is not None and self.entry_current.get_text() != self.password:
-				Error(self, "Incorrect password", "The password you entered as the current file password is incorrect.").run()
+				Error(self, _('Incorrect password'), _('The password you entered as the current file password is incorrect.')).run()
 
 			elif self.entry_new.get_text() != self.entry_confirm.get_text():
-				Error(self, "Passwords don't match", "The password and password confirmation you entered does not match.").run()
+				Error(self, _('Passwords don\'t match'), _('The password and password confirmation you entered does not match.')).run()
 
 			else:
 				password = self.entry_new.get_text()
@@ -640,8 +690,8 @@ class PasswordChange(Password):
 
 				except ValueError, res:
 					response = Warning(
-						self, "Use insecure password?",
-						"The password you entered is not secure; " + str(res).lower() + ". Are you sure you want to use it?",
+						self, _('Use insecure password?'),
+						_('The password you entered is not secure; %s. Are you sure you want to use it?') % str(res).lower(),
 						( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( gtk.STOCK_OK, gtk.RESPONSE_OK ) ), gtk.RESPONSE_CANCEL
 					).run()
 
@@ -658,15 +708,15 @@ class PasswordLock(Password):
 
 	def __init__(self, parent, password):
 		Password.__init__(
-			self, parent, "Enter password to unlock file",
-			"The current file has been locked, please enter the file password to unlock it.",
+			self, parent, _('Enter password to unlock file'),
+			_('The current file has been locked, please enter the file password to unlock it.'),
 			ui.STOCK_UNLOCK
 		)
 
 		self.get_button(1).set_label(gtk.STOCK_QUIT)
 
 		self.password = password
-		self.entry_password = self.add_entry("Password")
+		self.entry_password = self.add_entry(_('Password'))
 
 
 	def run(self):
@@ -686,7 +736,7 @@ class PasswordLock(Password):
 					break
 
 				else:
-					Error(self, "Incorrect password", "The password you entered was not correct, please try again.").run()
+					Error(self, _('Incorrect password'), _('The password you entered was not correct, please try again.')).run()
 
 			except CancelError:
 				if self.get_button(1).get_property("sensitive") == True:
@@ -705,12 +755,12 @@ class PasswordOpen(Password):
 
 	def __init__(self, parent, filename):
 		Password.__init__(
-			self, parent, "Enter file password",
-			"The file '%s' is encrypted. Please enter the file password to open it." % filename,
+			self, parent, _('Enter file password'),
+			_('The file \'%s\' is encrypted. Please enter the file password to open it.') % filename,
 			gtk.STOCK_OPEN
 		)
 
-		self.entry_password = self.add_entry("Password")
+		self.entry_password = self.add_entry(_('Password'))
 
 
 	def run(self):
@@ -733,13 +783,13 @@ class PasswordSave(Password):
 
 	def __init__(self, parent, filename):
 		Password.__init__(
-			self, parent, "Enter password for file",
-			"Please enter a password for the file '%s'. You will need this password to open the file at a later time." % filename,
+			self, parent, _('Enter password for file'),
+			_('Please enter a password for the file \'%s\'. You will need this password to open the file at a later time.') % filename,
 			gtk.STOCK_SAVE
 		)
 
-		self.entry_new		= self.add_entry("New password", ui.PasswordEntry())
-		self.entry_confirm	= self.add_entry("Confirm password")
+		self.entry_new		= self.add_entry(_('New password'), ui.PasswordEntry())
+		self.entry_confirm	= self.add_entry(_('Confirm password'))
 		self.entry_confirm.autocheck = False
 
 
@@ -752,10 +802,10 @@ class PasswordSave(Password):
 				raise CancelError
 
 			elif self.entry_new.get_text() != self.entry_confirm.get_text():
-				Error(self, "Passwords don't match", "The passwords you entered does not match.").run()
+				Error(self, _('Passwords don\'t match'), _('The passwords you entered does not match.')).run()
 
 			elif len(self.entry_new.get_text()) == 0:
-				Error(self, "No password entered", "You must enter a password for the new data file.").run()
+				Error(self, _('No password entered'), _('You must enter a password for the new data file.')).run()
 
 			else:
 				password = self.entry_new.get_text()
@@ -765,8 +815,8 @@ class PasswordSave(Password):
 
 				except ValueError, res:
 					response = Warning(
-						self, "Use insecure password?",
-						"The password you entered is not secure; " + str(res).lower() + ". Are you sure you want to use it?",
+						self, _('Use insecure password?'),
+						_('The password you entered is not secure; %s. Are you sure you want to use it?') % str(res).lower(),
 						( ( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ), ( gtk.STOCK_OK, gtk.RESPONSE_OK ) ), gtk.RESPONSE_CANCEL
 					).run()
 
@@ -800,22 +850,22 @@ class EntryEdit(Utility):
 
 		# set up the ui
 		self.sect_meta		= self.add_section(title)
-		self.sect_fields	= self.add_section("Account Data")
+		self.sect_fields	= self.add_section(_('Account Data'))
 
 		self.entry_name = ui.Entry()
 		self.entry_name.set_width_chars(50)
-		self.tooltips.set_tip(self.entry_name, "The name of the entry")
-		self.sect_meta.append_widget("Name", self.entry_name)
+		self.tooltips.set_tip(self.entry_name, _('The name of the entry'))
+		self.sect_meta.append_widget(_('Name'), self.entry_name)
 
 		self.entry_desc = ui.Entry()
-		self.tooltips.set_tip(self.entry_desc, "A description of the entry")
-		self.sect_meta.append_widget("Description", self.entry_desc)
+		self.tooltips.set_tip(self.entry_desc, _('A description of the entry'))
+		self.sect_meta.append_widget(_('Description'), self.entry_desc)
 
 		self.dropdown = ui.EntryDropDown()
 		self.dropdown.connect("changed", lambda w: self.__setup_fieldsect(self.dropdown.get_active_type()().fields))
 		eventbox = ui.EventBox(self.dropdown)
-		self.tooltips.set_tip(eventbox, "The type of entry")
-		self.sect_meta.append_widget("Type", eventbox)
+		self.tooltips.set_tip(eventbox, _('The type of entry'))
+		self.sect_meta.append_widget(_('Type'), eventbox)
 
 		# populate the dialog with data
 		self.set_entry(e)
@@ -886,7 +936,7 @@ class EntryEdit(Utility):
 				e = self.get_entry()
 
 				if e.name == "":
-					Error(self, "Name not entered", "You must enter a name for the account").run()
+					Error(self, _('Name not entered'), _('You must enter a name for the account')).run()
 					continue
 
 				self.destroy()
@@ -926,16 +976,16 @@ class EntryRemove(Warning):
 	def __init__(self, parent, entries):
 
 		if len(entries) > 1:
-			title	= "Really remove the %i selected entries?" % len(entries)
-			text	= "By removing these entries you will also remove any entries they may contain."
+			title	= _('Really remove the %i selected entries?') % len(entries)
+			text	= _('By removing these entries you will also remove any entries they may contain.')
 
 		elif type(entries[0]) == entry.FolderEntry:
-			title	= "Really remove folder '%s'?" % entries[0].name
-			text	= "By removing this folder you will also remove all accounts and folders it contains."
+			title	= _('Really remove folder \'%s\'?') % entries[0].name
+			text	= _('By removing this folder you will also remove all accounts and folders it contains.')
 
 		else:
-			title	= "Really remove account '%s'?" % entries[0].name
-			text	= "Please confirm that you wish to remove this account."
+			title	= _('Really remove account \'%s\'?') % entries[0].name
+			text	= _('Please confirm that you wish to remove this account.')
 
 
 		Warning.__init__(
@@ -973,12 +1023,12 @@ class FolderEdit(Utility):
 
 		self.entry_name = ui.Entry()
 		self.entry_name.set_width_chars(25)
-		self.tooltips.set_tip(self.entry_name, "The name of the folder")
-		self.sect_folder.append_widget("Name", self.entry_name)
+		self.tooltips.set_tip(self.entry_name, _('The name of the folder'))
+		self.sect_folder.append_widget(_('Name'), self.entry_name)
 
 		self.entry_desc = ui.Entry()
-		self.tooltips.set_tip(self.entry_desc, "A description of the folder")
-		self.sect_folder.append_widget("Description", self.entry_desc)
+		self.tooltips.set_tip(self.entry_desc, _('A description of the folder'))
+		self.sect_folder.append_widget(_('Description'), self.entry_desc)
 
 		# populate the dialog with data
 		self.set_entry(e)
@@ -1004,7 +1054,7 @@ class FolderEdit(Utility):
 				e = self.get_entry()
 
 				if e.name == "":
-					Error(self, "Name not entered", "You must enter a name for the folder").run()
+					Error(self, _('Name not entered'), _('You must enter a name for the folder')).run()
 					continue
 
 				self.destroy()
@@ -1028,25 +1078,33 @@ class FolderEdit(Utility):
 
 ##### MISCELLANEOUS DIALOGS #####
 
-class About(gnome.ui.About):
+class About(gtk.AboutDialog):
 	"About dialog"
 
 	def __init__(self, parent):
-		gnome.ui.About.__init__(
-			self, config.APPNAME, config.VERSION, config.COPYRIGHT,
-			""""%s"\n\nRevelation is a password manager for the GNOME 2 desktop""" % config.RELNAME,
-			( config.AUTHOR, ), None, "",
-			parent.render_icon(ui.STOCK_REVELATION, gtk.ICON_SIZE_DIALOG)
-		)
+		gtk.AboutDialog.__init__(self)
 
-		if parent is not None:
+		if isinstance(parent, gtk.Window):
 			self.set_transient_for(parent)
 
+		self.set_name(config.APPNAME)
+		self.set_version(config.VERSION)
+		self.set_copyright(config.COPYRIGHT)
+		self.set_comments(('"%s"\n\n' + _('A password manager for the GNOME desktop')) % config.RELNAME)
+		self.set_license(config.LICENSE)
+		self.set_website(config.URL)
+		self.set_authors(config.AUTHORS)
+		self.set_artists(config.ARTISTS)
+		self.set_logo(parent.render_icon(ui.STOCK_REVELATION, gtk.ICON_SIZE_DIALOG))
+
 
 	def run(self):
 		"Displays the dialog"
 
 		self.show_all()
+		gtk.AboutDialog.run(self)
+
+		self.destroy()
 
 
 
@@ -1055,8 +1113,8 @@ class Exception(Error):
 
 	def __init__(self, parent, traceback):
 		Error.__init__(
-			self, parent, "Unknown error",
-			"An unknown error occured. Please report the text below to the Revelation developers, along with what you were doing that may have caused the error. You may attempt to continue running Revelation, but it may behave unexpectedly.",
+			self, parent, _('Unknown error'),
+			_('An unknown error occured. Please report the text below to the Revelation developers, along with what you were doing that may have caused the error. You may attempt to continue running Revelation, but it may behave unexpectedly.'),
 			( ( gtk.STOCK_QUIT, gtk.RESPONSE_CANCEL ), ( ui.STOCK_CONTINUE, gtk.RESPONSE_OK ) )
 		)
 
@@ -1077,26 +1135,27 @@ class Exception(Error):
 class PasswordChecker(Utility):
 	"A password checker dialog"
 
-	def __init__(self, parent):
+	def __init__(self, parent, cfg = None, clipboard = None):
 		Utility.__init__(
-			self, parent, "Password Checker",
+			self, parent, _('Password Checker'),
 			( ( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE ), )
 		)
 
+		self.cfg = cfg
 		self.set_modal(False)
 		self.set_size_request(300, -1)
 
-		self.section = self.add_section("Password Checker")
+		self.section = self.add_section(_('Password Checker'))
 
-		self.entry = ui.Entry()
+		self.entry = ui.PasswordEntry(None, cfg, clipboard)
+		self.entry.autocheck = False
 		self.entry.set_width_chars(40)
 		self.entry.connect("changed", self.__cb_changed)
-		self.tooltips.set_tip(self.entry, "Enter a password to check")
-		self.section.append_widget("Password", self.entry)
+		self.tooltips.set_tip(self.entry, _('Enter a password to check'))
+		self.section.append_widget(_('Password'), self.entry)
 
-		self.result = ui.ImageLabel("Enter a password to check", ui.STOCK_UNKNOWN)
-		self.result.set(0, 0.5, 0, 0)
-		self.tooltips.set_tip(self.result, "The result of the check")
+		self.result = ui.ImageLabel(_('Enter a password to check'), ui.STOCK_UNKNOWN, ui.ICON_SIZE_HEADLINE)
+		self.tooltips.set_tip(self.result, _('The result of the check'))
 		self.section.append_widget(None, self.result)
 
 		self.connect("response", self.__cb_response)
@@ -1110,19 +1169,19 @@ class PasswordChecker(Utility):
 		try:
 			if len(password) == 0:
 				icon	= ui.STOCK_UNKNOWN
-				result	= "Enter a password to check"
+				result	= _('Enter a password to check')
 
 			else:
 				util.check_password(password)
 				icon	= ui.STOCK_PASSWORD_STRONG
-				result	= "The password seems good"
+				result	= _('The password seems good')
 
 		except ValueError, result:
 			icon	= ui.STOCK_PASSWORD_WEAK
-			result = "The password " + str(result)
+			result = _('The password %s') % str(result)
 
 		self.result.set_text(result)
-		self.result.set_stock(icon, ui.ICON_SIZE_LABEL)
+		self.result.set_stock(icon, ui.ICON_SIZE_HEADLINE)
 
 
 	def __cb_response(self, widget, response):
@@ -1149,32 +1208,28 @@ class PasswordChecker(Utility):
 class PasswordGenerator(Utility):
 	"A password generator dialog"
 
-	def __init__(self, parent, cfg):
+	def __init__(self, parent, cfg, clipboard = None):
 		Utility.__init__(
-			self, parent, "Password Generator",
+			self, parent, _('Password Generator'),
 			( ( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE ), ( ui.STOCK_GENERATE, gtk.RESPONSE_OK ) )
 		)
 
 		self.config = cfg
 		self.set_modal(False)
 
-		self.section = self.add_section("Password Generator")
+		self.section = self.add_section(_('Password Generator'))
 
-		self.entry = ui.Entry()
+		self.entry = ui.PasswordEntry(None, cfg, clipboard)
+		self.entry.autocheck = False
 		self.entry.set_editable(False)
-		self.tooltips.set_tip(self.entry, "The generated password")
-		self.section.append_widget("Password", self.entry)
+		self.tooltips.set_tip(self.entry, _('The generated password'))
+		self.section.append_widget(_('Password'), self.entry)
 
 		self.spin_pwlen = ui.SpinEntry()
 		self.spin_pwlen.set_range(4, 256)
 		self.spin_pwlen.set_value(self.config.get("passwordgen/length"))
-		self.tooltips.set_tip(self.spin_pwlen, "The number of characters in generated passwords - 8 or more are recommended")
-		self.section.append_widget("Length", self.spin_pwlen)
-
-		self.check_ambiguous = ui.CheckButton("Avoid ambiguous characters")
-		self.check_ambiguous.set_active(self.config.get("passwordgen/avoid_ambiguous"))
-		self.tooltips.set_tip(self.check_ambiguous, "When enabled, generated passwords will not contain ambiguous characters - like 0 (zero) and O (capital o)")
-		self.section.append_widget(None, self.check_ambiguous)
+		self.tooltips.set_tip(self.spin_pwlen, ('The number of characters in generated passwords - 8 or more are recommended'))
+		self.section.append_widget(_('Length'), self.spin_pwlen)
 
 		self.connect("response", self.__cb_response)
 
@@ -1183,10 +1238,7 @@ class PasswordGenerator(Utility):
 		"Callback for dialog responses"
 
 		if response == gtk.RESPONSE_OK:
-			self.entry.set_text(util.generate_password(
-				self.spin_pwlen.get_value(),
-				self.check_ambiguous.get_active()
-			))
+			self.entry.set_text(util.generate_password(self.spin_pwlen.get_value()))
 
 		else:
 			self.destroy()
@@ -1203,248 +1255,6 @@ class PasswordGenerator(Utility):
 
 
 
-class Preferences(Utility):
-	"A preference dialog"
-
-	def __init__(self, parent, cfg):
-		Utility.__init__(self, parent, "Preferences")
-		self.config = cfg
-		self.set_modal(False)
-
-		self.notebook = ui.Notebook()
-		self.vbox.pack_start(self.notebook)
-
-		self.page_files = self.notebook.create_page("Files")
-		self.__init_section_startup(self.page_files)
-		self.__init_section_filehandling(self.page_files)
-
-		self.page_interface = self.notebook.create_page("Interface")
-		self.__init_section_doubleclick(self.page_interface)
-		self.__init_section_toolbar(self.page_interface)
-
-		self.page_password = self.notebook.create_page("Passwords")
-		self.__init_section_password_display(self.page_password)
-		self.__init_section_clipboard(self.page_password)
-		self.__init_section_pwgen(self.page_password)
-
-		self.page_gotocmd = self.notebook.create_page("Goto Commands")
-		self.__init_section_gotocmd(self.page_gotocmd)
-
-		self.connect("response", lambda w,d: self.destroy())
-
-
-	def __init_section_clipboard(self, page):
-		"Sets up the clipboard section"
-
-		self.section_clipboard = page.add_section("Clipboard")
-
-		# check-button for username
-		self.check_chain_username = ui.CheckButton("Also copy username when copying password")
-		ui.config_bind(self.config, "clipboard/chain_username", self.check_chain_username)
-
-		self.tooltips.set_tip(self.check_chain_username, "When the password is copied to clipboard, put the username before the password as a clipboard \"chain\"")
-		self.section_clipboard.append_widget(None, self.check_chain_username)
-
-
-	def __init_section_doubleclick(self, page):
-		"Sets up the doubleclick section"
-
-		self.section_doubleclick = page.add_section("Doubleclick Action")
-
-		# radio-button for go to
-		self.radio_doubleclick_goto = ui.RadioButton(None, "Go to account, if possible")
-		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_goto, "goto")
-
-		self.tooltips.set_tip(self.radio_doubleclick_goto, "Go to the account (open in external application) on doubleclick, if command is set and required data filled in")
-		self.section_doubleclick.append_widget(None, self.radio_doubleclick_goto)
-
-		# radio-button for edit
-		self.radio_doubleclick_edit = ui.RadioButton(self.radio_doubleclick_goto, "Edit account")
-		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_edit, "edit")
-
-		self.tooltips.set_tip(self.radio_doubleclick_edit, "Edit the account on doubleclick")
-		self.section_doubleclick.append_widget(None, self.radio_doubleclick_edit)
-
-		# radio-button for copy
-		self.radio_doubleclick_copy = ui.RadioButton(self.radio_doubleclick_goto, "Copy password to clipboard")
-		ui.config_bind(self.config, "behavior/doubleclick", self.radio_doubleclick_copy, "copy")
-
-		self.tooltips.set_tip(self.radio_doubleclick_copy, "Copy the password to clipboard on doubleclick")
-		self.section_doubleclick.append_widget(None, self.radio_doubleclick_copy)
-
-
-	def __init_section_filehandling(self, page):
-		"Sets up the file section"
-
-		self.section_filehandling = page.add_section("File Handling")
-
-		# check-button for autosave
-		self.check_autosave = ui.CheckButton("Autosave data when changed")
-		ui.config_bind(self.config, "file/autosave", self.check_autosave)
-
-		self.tooltips.set_tip(self.check_autosave, "Automatically saves the data file when an entry is added, modified or removed")
-		self.section_filehandling.append_widget(None, self.check_autosave)
-
-		# autolock file
-		self.check_autolock = ui.CheckButton("Autolock file when inactive for")
-		ui.config_bind(self.config, "file/autolock", self.check_autolock)
-		self.check_autolock.connect("toggled", lambda w: self.spin_autolock_timeout.set_sensitive(w.get_active()))
-		self.tooltips.set_tip(self.check_autolock, "Automatically lock the data file after a period of inactivity")
-
-		self.spin_autolock_timeout = ui.SpinEntry()
-		self.spin_autolock_timeout.set_range(1, 120)
-		self.spin_autolock_timeout.set_sensitive(self.check_autolock.get_active())
-		ui.config_bind(self.config, "file/autolock_timeout", self.spin_autolock_timeout)
-		self.tooltips.set_tip(self.spin_autolock_timeout, "The period of inactivity before locking the file, in minutes")
-
-		hbox = ui.HBox()
-		hbox.set_spacing(3)
-		hbox.pack_start(self.check_autolock)
-		hbox.pack_start(self.spin_autolock_timeout)
-		hbox.pack_start(ui.Label("minutes"))
-		self.section_filehandling.append_widget(None, hbox)
-
-
-	def __init_section_gotocmd(self, page):
-		"Sets up the goto command section"
-
-		self.section_gotocmd = page.add_section("Goto Commands")
-
-		for entrytype in entry.ENTRYLIST:
-			if entrytype == entry.FolderEntry:
-				continue
-
-			e = entrytype()
-
-			widget = ui.Entry()
-			ui.config_bind(self.config, "launcher/%s" % e.id, widget)
-
-			tooltip = "Goto command for %s accounts. The following expansion variables can be used:\n\n" % e.typename
-
-			for field in e.fields:
-				tooltip += "%%%s: %s\n" % ( field.symbol, field.name )
-
-			tooltip += "\n"
-			tooltip += "%%: a % sign\n"
-			tooltip += "%?x: optional expansion variable\n"
-			tooltip += "%(...%): optional substring expansion"
-
-			self.tooltips.set_tip(widget, tooltip)
-			self.section_gotocmd.append_widget(e.typename, widget)
-
-
-	def __init_section_password_display(self, page):
-		"Sets up the password display section"
-
-		self.section_password_display = page.add_section("Password Display")
-
-		# show passwords checkbutton
-		self.check_show_passwords = ui.CheckButton("Show passwords and other secrets")
-		ui.config_bind(self.config, "view/passwords", self.check_show_passwords)
-
-		self.tooltips.set_tip(self.check_show_passwords, "Display passwords and other secrets, such as PIN codes (otherwise, hide with ******)")
-		self.section_password_display.append_widget(None, self.check_show_passwords)
-
-
-	def __init_section_pwgen(self, page):
-		"Sets up the password generator section"
-
-		self.section_pwgen = page.add_section("Password Generator")
-
-		# password length spinbutton
-		self.spin_pwlen = ui.SpinEntry()
-		self.spin_pwlen.set_range(4, 32)
-		ui.config_bind(self.config, "passwordgen/length", self.spin_pwlen)
-
-		self.tooltips.set_tip(self.spin_pwlen, "The number of characters in generated passwords - 8 or more are recommended")
-		self.section_pwgen.append_widget("Password length", self.spin_pwlen)
-
-		# checkbox for avoiding ambiguous characters
-		self.check_ambiguous = ui.CheckButton("Avoid ambiguous characters")
-		ui.config_bind(self.config, "passwordgen/avoid_ambiguous", self.check_ambiguous)
-
-		self.tooltips.set_tip(self.check_ambiguous, "When enabled, generated passwords will not contain ambiguous characters - like 0 (zero) and O (capital o)")
-		self.section_pwgen.append_widget(None, self.check_ambiguous)
-
-
-	def __init_section_startup(self, page):
-		"Sets up the startup section"
-
-		self.section_startup = page.add_section("Startup")
-
-		# check-button for autoloading a file
-		self.check_autoload = ui.CheckButton("Open file on startup")
-		ui.config_bind(self.config, "file/autoload", self.check_autoload)
-		self.check_autoload.connect("toggled", lambda w: self.entry_autoload_file.set_sensitive(w.get_active()))
-
-		self.tooltips.set_tip(self.check_autoload, "When enabled, a file will be opened when the program is started")
-		self.section_startup.append_widget(None, self.check_autoload)
-
-		# entry for file to autoload
-		self.entry_autoload_file = ui.FileEntry("Select File to Automatically Open")
-		ui.config_bind(self.config, "file/autoload_file", self.entry_autoload_file)
-		self.entry_autoload_file.set_sensitive(self.check_autoload.get_active())
-
-		eventbox = ui.EventBox(self.entry_autoload_file)
-		self.tooltips.set_tip(eventbox, "A file to be opened when the program is started")
-		self.section_startup.append_widget("File to open", eventbox)
-
-
-	def __init_section_toolbar(self, page):
-		"Sets up the toolbar section"
-
-		self.section_toolbar = page.add_section("Toolbar Style")
-
-		# radio-button for desktop default
-		self.radio_toolbar_desktop = ui.RadioButton(None, "Use desktop default")
-		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_desktop, "desktop")
-
-		self.tooltips.set_tip(self.radio_toolbar_desktop, "Show toolbar items with default style")
-		self.section_toolbar.append_widget(None, self.radio_toolbar_desktop)
-
-		# radio-button for icons and text
-		self.radio_toolbar_both = ui.RadioButton(self.radio_toolbar_desktop, "Show icons and text")
-		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_both, "both")
-
-		self.tooltips.set_tip(self.radio_toolbar_both, "Show toolbar items with both icons and text")
-		self.section_toolbar.append_widget(None, self.radio_toolbar_both)
-
-		# radio-button for icons and important text
-		self.radio_toolbar_bothhoriz = ui.RadioButton(self.radio_toolbar_desktop, "Show icons and important text")
-		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_bothhoriz, "both-horiz")
-
-		self.tooltips.set_tip(self.radio_toolbar_bothhoriz, "Show toolbar items with text beside important icons")
-		self.section_toolbar.append_widget(None, self.radio_toolbar_bothhoriz)
-
-		# radio-button for icons only
-		self.radio_toolbar_icons = ui.RadioButton(self.radio_toolbar_desktop, "Show icons only")
-		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_icons, "icons")
-
-		self.tooltips.set_tip(self.radio_toolbar_icons, "Show toolbar items with icons only")
-		self.section_toolbar.append_widget(None, self.radio_toolbar_icons)
-
-		# radio-button for text only
-		self.radio_toolbar_text = ui.RadioButton(self.radio_toolbar_desktop, "Show text only")
-		ui.config_bind(self.config, "view/toolbar_style", self.radio_toolbar_text, "text")
-
-		self.tooltips.set_tip(self.radio_toolbar_text, "Show toolbar items with text only")
-		self.section_toolbar.append_widget(None, self.radio_toolbar_text)
-
-
-	def run(self):
-		"Runs the preference dialog"
-
-		self.show_all()
-
-		if EVENT_FILTER != None:
-			self.window.add_filter(EVENT_FILTER)
-
-		# for some reason, gtk crashes on close-by-escape if we don't do this
-		self.get_button(0).grab_focus()
-		self.notebook.grab_focus()
-
-
-
 ##### FUNCTIONS #####
 
 def create_unique(dialog, *args):
