@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: io.py 477 2006-02-02 22:24:40Z erikg $
+# $Id: io.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for IO-related functionality
 #
@@ -25,13 +25,7 @@
 
 import datahandler
 
-import gobject, os.path, re
-
-try:
-	import gnomevfs
-
-except ImportError:
-	import gnome.vfs as gnomevfs
+import gnomevfs, gobject, os.path, re
 
 
 
@@ -129,7 +123,9 @@ class DataFile(gobject.GObject):
 
 		self.__monitor_stop()
 		file_write(file, self.__handler.export_data(entrystore, password))
-		self.__monitor(file)
+
+		# need to use idle_add() to avoid notifying about current save
+		gobject.idle_add(lambda: self.__monitor(file))
 
 		self.set_password(password)
 		self.set_file(file)
@@ -205,7 +201,7 @@ def file_normpath(file):
 	"Normalizes a file path"
 
 	if file in ( None, "" ):
-		return None
+		return ""
 
 	file = re.sub("^file:/{,2}", "", file)
 	file = os.path.expanduser(file)
