@@ -1,7 +1,7 @@
 #
-# Revelation 0.4.7 - a password manager for GNOME 2
+# Revelation - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: ui.py 476 2006-02-02 19:28:56Z erikg $
+# $Id: ui.py 602 2007-01-03 08:06:28Z erikg $
 #
 # Module for UI functionality
 #
@@ -25,8 +25,9 @@
 
 import config, data, dialog, entry, io, util
 
-import bonobo.ui, gobject, gtk, gtk.gdk, gnome.ui, os, pango, pwd, time, xml.dom.minidom
-from xml.parsers.expat import ExpatError
+import bonobo.ui, gettext, gobject, gtk, gtk.gdk, gnome.ui, os, pango, pwd, time
+
+_ = gettext.gettext
 
 
 STOCK_CONTINUE			= "revelation-continue"
@@ -41,7 +42,6 @@ STOCK_LOCK			= "revelation-lock"
 STOCK_NEW_ENTRY			= "revelation-new-entry"
 STOCK_NEW_FOLDER		= "revelation-new-folder"
 STOCK_NEXT			= "revelation-next"
-STOCK_OVERWRITE			= "revelation-overwrite"
 STOCK_PASSWORD_CHANGE		= "revelation-password-change"
 STOCK_PASSWORD_CHECK		= "revelation-password-check"
 STOCK_PASSWORD_STRONG		= "revelation-password-strong"
@@ -49,9 +49,11 @@ STOCK_PASSWORD_WEAK		= "revelation-password-weak"
 STOCK_PREVIOUS			= "revelation-previous"
 STOCK_RELOAD			= "revelation-reload"
 STOCK_REMOVE			= "revelation-remove"
+STOCK_REPLACE			= "revelation-replace"
 STOCK_UNKNOWN			= "revelation-unknown"
 STOCK_UNLOCK			= "revelation-unlock"
 STOCK_UPDATE			= "revelation-update"
+STOCK_WARNING			= "revelation-warning"
 
 
 STOCK_ENTRY_FOLDER		= "revelation-account-folder"
@@ -75,9 +77,55 @@ ICON_SIZE_APPLET		= gtk.ICON_SIZE_LARGE_TOOLBAR
 ICON_SIZE_DATAVIEW		= gtk.ICON_SIZE_LARGE_TOOLBAR
 ICON_SIZE_DROPDOWN		= gtk.ICON_SIZE_SMALL_TOOLBAR
 ICON_SIZE_ENTRY			= gtk.ICON_SIZE_MENU
-ICON_SIZE_LABEL			= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_FALLBACK		= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_HEADLINE		= gtk.ICON_SIZE_LARGE_TOOLBAR
+ICON_SIZE_LABEL			= gtk.ICON_SIZE_MENU
 ICON_SIZE_LOGO			= gtk.ICON_SIZE_DND
-ICON_SIZE_TREEVIEW		= gtk.ICON_SIZE_SMALL_TOOLBAR
+ICON_SIZE_TREEVIEW		= gtk.ICON_SIZE_MENU
+
+STOCK_ICONS			= (
+	( STOCK_REVELATION,		"revelation",			( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU )),
+	( STOCK_REVELATION_LOCKED,	"revelation-locked",		( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU )),
+	( STOCK_ENTRY_CREDITCARD,	"stock_creditcard",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_CRYPTOKEY,	"stock_keyring",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_DATABASE,		"stock_data-sources",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_DOOR,		"stock_exit",			( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_EMAIL,		"stock_mail",			( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_FTP,		"system-file-manager",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_GENERIC,		"stock_lock",			( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_PHONE,		"stock_cell-phone",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_SHELL,		"gnome-terminal",		( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_WEBSITE,		"stock_hyperlink-toolbar",	( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_FOLDER,		"stock_folder",			( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+	( STOCK_ENTRY_FOLDER_OPEN,	"stock_folder",			( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW )),
+)
+
+STOCK_ITEMS = (
+	( STOCK_CONTINUE,		_('_Continue'),			"stock_test-mode" ),
+	( STOCK_DISCARD,		_('_Discard'),			gtk.STOCK_DELETE ),
+	( STOCK_EDIT,			_('_Edit'),			"stock_edit" ),
+	( STOCK_EXPORT,			_('_Export'),			gtk.STOCK_EXECUTE ),
+	( STOCK_FOLDER,			'',				"stock_folder" ),
+	( STOCK_GENERATE,		_('_Generate'),			gtk.STOCK_EXECUTE ),
+	( STOCK_GOTO,			_('_Go to'),			gtk.STOCK_JUMP_TO ),
+	( STOCK_IMPORT,			_('_Import'),			gtk.STOCK_CONVERT ),
+	( STOCK_LOCK,			_('_Lock'),			"stock_lock" ),
+	( STOCK_NEW_ENTRY,		_('_Add Entry'),		gtk.STOCK_ADD ),
+	( STOCK_NEW_FOLDER,		_('_Add Folder'),		"stock_folder" ),
+	( STOCK_NEXT,			_('Next'),			gtk.STOCK_GO_DOWN ),
+	( STOCK_PASSWORD_CHANGE,	_('_Change'),			"stock_lock-ok" ),
+	( STOCK_PASSWORD_CHECK,		_('_Check'),			"stock_lock-ok" ),
+	( STOCK_PASSWORD_STRONG,	'',				"stock_lock-ok" ),
+	( STOCK_PASSWORD_WEAK,		'',				"stock_lock-broken" ),
+	( STOCK_PREVIOUS,		_('Previous'),			gtk.STOCK_GO_UP ),
+	( STOCK_RELOAD,			_('_Reload'),			gtk.STOCK_REFRESH ),
+	( STOCK_REMOVE,			_('Re_move'),			gtk.STOCK_DELETE ),
+	( STOCK_REPLACE,		_('_Replace'),			gtk.STOCK_SAVE_AS ),
+	( STOCK_UNKNOWN,		_('Unknown'),			gtk.STOCK_DIALOG_QUESTION ),
+	( STOCK_UNLOCK,			_('_Unlock'),			"stock_lock-open" ),
+	( STOCK_UPDATE,			_('_Update'),			"stock_edit" ),
+	( STOCK_WARNING,		'',				"stock_dialog-warning" ),
+)
 
 
 
@@ -121,6 +169,11 @@ def config_bind(cfg, key, widget, data = None):
 		cb_get	= lambda c,v,w:	w.set_text(v)
 		cb_set	= lambda w,k:	cfg.set(k, w.get_text())
 
+	elif isinstance(widget, gtk.FileChooserButton):
+		signal	= "selection-changed"
+		cb_get	= lambda c,v,w: w.set_filename(io.file_normpath(v))
+		cb_set	= lambda w,k:	cfg.set(k, io.file_normpath(w.get_filename()))
+
 	else:
 		raise config.ConfigError
 
@@ -179,9 +232,12 @@ def generate_field_edit_widget(field, cfg = None, userdata = None):
 class Alignment(gtk.Alignment):
 	"A container bin"
 
-	def __init__(self, xalign = 0, yalign = 0, xscale = 0, yscale = 0):
+	def __init__(self, widget = None, xalign = 0, yalign = 0, xscale = 0, yscale = 0):
 		gtk.Alignment.__init__(self, xalign, yalign, xscale, yscale)
 
+		if widget != None:
+			self.add(widget)
+
 
 
 class HBox(gtk.HBox):
@@ -292,6 +348,23 @@ class ScrolledWindow(gtk.ScrolledWindow):
 
 
 
+class Table(gtk.Table):
+	"A table"
+
+	def __init__(self, rows = 1, cols = 1, homogenous = False):
+		gtk.Table.__init__(self, rows, cols, homogenous)
+
+		self.set_row_spacings(3)
+		self.set_col_spacings(6)
+
+
+	def attach(self, widget, x, y, colspan = 1, rowspan = 1, xoptions = gtk.FILL, yoptions = 0):
+		"Attaches a widget to the table"
+
+		gtk.Table.attach(self, widget, x, x + colspan, y, y + colspan, xoptions, yoptions)
+
+
+
 class Toolbar(gtk.Toolbar):
 	"A Toolbar subclass"
 
@@ -398,20 +471,17 @@ class Image(gtk.Image):
 
 
 
-class ImageLabel(gtk.Alignment):
+class ImageLabel(HBox):
 	"A label with an image"
 
 	def __init__(self, text = None, stock = None, size = ICON_SIZE_LABEL):
-		gtk.Alignment.__init__(self, 0.5, 0.5, 0, 0)
-
-		self.hbox = HBox()
-		self.add(self.hbox)
+		HBox.__init__(self)
 
 		self.image = Image()
-		self.hbox.pack_start(self.image, False, False)
+		self.pack_start(self.image, False, True)
 
-		self.label = Label("", gtk.JUSTIFY_CENTER)
-		self.hbox.pack_start(self.label)
+		self.label = Label(text)
+		self.pack_start(self.label)
 
 		if text != None:
 			self.set_text(text)
@@ -420,6 +490,12 @@ class ImageLabel(gtk.Alignment):
 			self.set_stock(stock, size)
 
 
+	def set_ellipsize(self, ellipsize):
+		"Sets label ellisization"
+
+		self.label.set_ellipsize(ellipsize)
+
+
 	def set_stock(self, stock, size):
 		"Sets the image"
 
@@ -505,7 +581,7 @@ class PasswordLabel(EventBox):
 		elif data.button == 3:
 			menu = Menu()
 
-			menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
+			menuitem = ImageMenuItem(gtk.STOCK_COPY, _('Copy password'))
 			menuitem.connect("activate", lambda w: self.clipboard.set(self.password, True))
 			menu.append(menuitem)
 
@@ -515,6 +591,12 @@ class PasswordLabel(EventBox):
 			return True
 
 
+	def set_ellipsize(self, ellipsize):
+		"Sets ellipsize for the label"
+
+		self.label.set_ellipsize(ellipsize)
+
+
 	def show_password(self, show = True):
 		"Sets whether to display the password"
 
@@ -634,14 +716,14 @@ class FileEntry(HBox):
 	def __init__(self, title = None, file = None, type = gtk.FILE_CHOOSER_ACTION_OPEN):
 		HBox.__init__(self)
 
-		self.title = title is not None and title or "Select File"
+		self.title = title is not None and title or _('Select File')
 		self.type = type
 
 		self.entry = Entry()
 		self.entry.connect("changed", lambda w: self.emit("changed"))
 		self.pack_start(self.entry)
 
-		self.button = Button("Browse...", self.__cb_filesel)
+		self.button = Button(_('Browse...'), self.__cb_filesel)
 		self.pack_start(self.button, False, False)
 
 		if file is not None:
@@ -715,9 +797,8 @@ class IconEntry(Alignment):
 		self.iconebox	= EventBox()
 		self.iconebox.set_visible_window(False)
 
-		self.iconalign	= Alignment(1.0, 0.5, 0, 0)
+		self.iconalign	= Alignment(self.iconebox, 1.0, 0.5, 0, 0)
 		self.iconalign.set_padding(1, 1, 0, 2)
-		self.iconalign.add(self.iconebox)
 
 		# connect signals
 		self.connect("expose-event", self.__cb_expose)
@@ -814,12 +895,24 @@ class IconEntry(Alignment):
 		return self.entry.get_text()
 
 
+	def grab_focus(self):
+		"Wrapper for gtk.Entry.grab_focus()"
+
+		return self.entry.grab_focus()
+
+
 	def remove_icon(self):
 		"Removes the icon from the entry"
 
 		self.set_icon(None, "")
 
 
+	def set_editable(self, editable):
+		"Wrapper for gtk.Entry.set_editable()"
+
+		self.entry.set_editable(editable)
+
+
 	def set_icon(self, stock, tooltip = ""):
 		"Sets the icon for the entry"
 
@@ -851,6 +944,12 @@ class IconEntry(Alignment):
 		self.entry.set_text(text)
 
 
+	def set_width_chars(self, width):
+		"Wrapper for gtk.Entry.set_width_chars"
+
+		self.entry.set_width_chars(width)
+
+
 	def set_visibility(self, visibility):
 		"Wrapper for the entry"
 
@@ -897,17 +996,17 @@ class PasswordEntry(IconEntry):
 				util.check_password(password)
 
 			except ValueError, reason:
-				self.set_password_strong(False, "The password %s" % str(reason))
+				self.set_password_strong(False, _('The password %s') % str(reason))
 
 			else:
-				self.set_password_strong(True, "The password seems good")
+				self.set_password_strong(True, _('The password seems good'))
 
 
 	def __cb_popup(self, widget, menu):
 		"Populates the popup menu"
 
 		if self.clipboard != None:
-			menuitem = ImageMenuItem(gtk.STOCK_COPY, "Copy password")
+			menuitem = ImageMenuItem(gtk.STOCK_COPY, _('Copy password'))
 			menuitem.connect("activate", lambda w: self.clipboard.set(self.get_text(), True))
 
 			menu.insert(menuitem, 2)
@@ -932,7 +1031,7 @@ class PasswordEntryGenerate(HBox):
 		self.pwentry = PasswordEntry(password, cfg, clipboard)
 		self.pack_start(self.pwentry)
 
-		self.button = Button("Generate", lambda w: self.generate())
+		self.button = Button(_('Generate'), lambda w: self.generate())
 		self.pack_start(self.button, False, False)
 
 		self.entry = self.pwentry.entry
@@ -941,10 +1040,7 @@ class PasswordEntryGenerate(HBox):
 	def generate(self):
 		"Generates a password for the entry"
 
-		length = self.config.get("passwordgen/length")
-		avoid_ambiguous = self.config.get("passwordgen/avoid_ambiguous")
-		password = util.generate_password(length, avoid_ambiguous)
-
+		password = util.generate_password(self.config.get("passwordgen/length"))
 		self.pwentry.set_text(password)
 
 
@@ -1073,7 +1169,7 @@ class EntryDropDown(DropDown):
 
 		for e in entry.ENTRYLIST:
 			if e != entry.FolderEntry:
-				self.append_item(e.typename, e.icon, e)
+				self.append_item(e().typename, e().icon, e)
 
 
 	def get_active_type(self):
@@ -1095,6 +1191,34 @@ class EntryDropDown(DropDown):
 				self.set_active(i)
 
 
+class FileButton(gtk.FileChooserButton):
+	"A file chooser button"
+
+	def __init__(self, title = None, file = None, type = gtk.FILE_CHOOSER_ACTION_OPEN):
+		gtk.FileChooserButton.__init__(self, title)
+		self.set_action(type)
+		self.set_local_only(False)
+
+		if file != None:
+			self.set_filename(file)
+
+
+	def get_filename(self):
+		"Gets the filename"
+
+		return io.file_normpath(self.get_uri())
+
+
+	def set_filename(self, filename):
+		"Sets the filename"
+
+		filename = io.file_normpath(filename)
+
+		if filename != io.file_normpath(self.get_filename()):
+			gtk.FileChooserButton.set_filename(self, filename)
+
+
+
 class LinkButton(gnome.ui.HRef):
 	"A link button"
 
@@ -1102,6 +1226,20 @@ class LinkButton(gnome.ui.HRef):
 		gnome.ui.HRef.__init__(self, url, label)
 		self.set_alignment(0, 0.5)
 
+		self.label = self.get_children()[0]
+
+
+	def set_ellipsize(self, ellipsize):
+		"Sets ellipsize for label"
+
+		self.label.set_ellipsize(ellipsize)
+
+
+	def set_justify(self, justify):
+		"Sets justify for label"
+
+		self.label.set_justify(justify)
+
 
 class RadioButton(gtk.RadioButton):
 	"A radio button"
@@ -1180,7 +1318,9 @@ class TreeView(gtk.TreeView):
 		if data.button == 1 and data.type == gtk.gdk._2BUTTON_PRESS and path != None:
 			iter = self.model.get_iter(path[0])
 			self.toggle_expanded(iter)
-			self.emit("doubleclick", iter)
+
+			if iter != None:
+				self.emit("doubleclick", iter)
 
 		# display popup on right-click
 		elif data.button == 3:
@@ -1429,83 +1569,58 @@ class ItemFactory(gtk.IconFactory):
 		if config.DIR_ICONS not in self.theme.get_search_path():
 			self.theme.append_search_path(config.DIR_ICONS)
 
-		self.load_stock_icon(STOCK_REVELATION, "revelation", ( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU ))
-		self.load_stock_icon(STOCK_REVELATION_LOCKED, "revelation-locked", ( ICON_SIZE_APPLET, ICON_SIZE_LOGO, gtk.ICON_SIZE_DIALOG, gtk.ICON_SIZE_MENU ))
-
-		self.__init_entryicons()
+		self.__init_icons()
 		self.__init_items()
 
 		self.theme.connect("changed", self.__cb_theme_changed)
 
 
-	def __init_entryicons(self):
-		"Loads entry icons"
+	def __init_icons(self):
+		"Loads stock icons"
 
-		icons = {
-			STOCK_ENTRY_CREDITCARD		: "stock_creditcard",
-			STOCK_ENTRY_CRYPTOKEY		: "stock_keyring",
-			STOCK_ENTRY_DATABASE		: "stock_data-sources",
-			STOCK_ENTRY_DOOR		: "stock_exit",
-			STOCK_ENTRY_EMAIL		: "stock_mail",
-			STOCK_ENTRY_FTP			: "gnome-ftp",
-			STOCK_ENTRY_GENERIC		: "stock_lock",
-			STOCK_ENTRY_PHONE		: "stock_cell-phone",
-			STOCK_ENTRY_SHELL		: "gnome-terminal",
-			STOCK_ENTRY_WEBSITE		: "stock_hyperlink-toolbar",
-			STOCK_ENTRY_FOLDER		: "gnome-fs-directory",
-			STOCK_ENTRY_FOLDER_OPEN		: "gnome-fs-directory-accept"
-		}
-
-		if self.theme.has_icon(icons[STOCK_ENTRY_FOLDER]) == False:
-			icons[STOCK_ENTRY_FOLDER] = "revelation-fallback-folder"
-
-		if self.theme.has_icon(icons[STOCK_ENTRY_FOLDER_OPEN]) == False:
-			icons[STOCK_ENTRY_FOLDER_OPEN] = "revelation-fallback-folder-open"
-
-		for id, name in icons.items():
-			self.load_stock_icon(id, name, ( ICON_SIZE_DATAVIEW, ICON_SIZE_DROPDOWN, ICON_SIZE_ENTRY, ICON_SIZE_TREEVIEW ))
+		for id, icon, sizes in STOCK_ICONS:
+			self.create_stock_icon(id, icon, sizes)
 
 
 	def __init_items(self):
 		"Creates stock items"
 
-		items = (
-			( STOCK_CONTINUE,	"_Continue",	"stock_test-mode" ),
-			( STOCK_DISCARD,	"_Discard",	gtk.STOCK_DELETE ),
-			( STOCK_EDIT,		"_Edit",	"stock_edit" ),
-			( STOCK_EXPORT,		"_Export",	gtk.STOCK_EXECUTE ),
-			( STOCK_FOLDER,		"",		"stock_folder" ),
-			( STOCK_GENERATE,	"_Generate",	gtk.STOCK_EXECUTE ),
-			( STOCK_GOTO,		"_Go to",	gtk.STOCK_JUMP_TO ),
-			( STOCK_IMPORT,		"_Import",	gtk.STOCK_CONVERT ),
-			( STOCK_LOCK,		"_Lock",	"stock_lock" ),
-			( STOCK_NEW_ENTRY,	"_Add Entry",	gtk.STOCK_ADD ),
-			( STOCK_NEW_FOLDER,	"_Add Folder",	"stock_folder" ),
-			( STOCK_NEXT,		"Next",		gtk.STOCK_GO_DOWN ),
-			( STOCK_OVERWRITE,	"_Overwrite",	gtk.STOCK_SAVE_AS ),
-			( STOCK_PASSWORD_CHANGE,"_Change",	"stock_lock-ok" ),
-			( STOCK_PASSWORD_CHECK,	"_Check",	"stock_lock-ok" ),
-			( STOCK_PASSWORD_STRONG,"",		"stock_lock-ok" ),
-			( STOCK_PASSWORD_WEAK,	"",		"stock_lock-broken" ),
-			( STOCK_PREVIOUS,	"Previous",	gtk.STOCK_GO_UP ),
-			( STOCK_RELOAD,		"_Reload",	gtk.STOCK_REFRESH ),
-			( STOCK_REMOVE,		"Re_move",	gtk.STOCK_DELETE ),
-			( STOCK_UNKNOWN,	"Unknown",	gtk.STOCK_DIALOG_QUESTION ),
-			( STOCK_UNLOCK,		"_Unlock",	"stock_lock-open" ),
-			( STOCK_UPDATE,		"_Update",	"stock_edit" ),
-		)
-
-		for id, name, icon in items:
+		for id, name, icon in STOCK_ITEMS:
 			self.create_stock_item(id, name, icon)
 
 
 	def __cb_theme_changed(self, widget, data = None):
 		"Callback for changed theme"
 
-		self.__init_entryicons()
+		self.__init_icons()
 		self.__init_items()
 
 
+	def create_stock_icon(self, id, icon, sizes):
+		"Creates a stock icon from a different stock icon"
+
+		iconset = gtk.IconSet()
+		self.add(id, iconset)
+
+		if self.theme.has_icon(icon) == False:
+			return
+
+		# load icons (the dict.fromkeys() thing is to remove duplicates)
+		for size in dict.fromkeys(sizes).keys():
+			source = self.get_iconsource(icon, size)
+
+			if source != None:
+				iconset.add_source(source)
+
+
+		# load fallback icon if none were found
+		if len(iconset.get_sizes()) == 0:
+			source = self.get_iconsource(icon, ICON_SIZE_FALLBACK, True)
+
+			if source != None:
+				iconset.add_source(source)
+
+
 	def create_stock_item(self, id, name, icon = None):
 		"Creates a stock item"
 
@@ -1515,46 +1630,82 @@ class ItemFactory(gtk.IconFactory):
 			pass
 
 		elif gtk.stock_lookup(icon) is not None:
-			iconset = self.parent.get_style().lookup_icon_set(icon)
-			self.add(id, iconset)
+			self.add(id, self.parent.get_style().lookup_icon_set(icon))
 
 		else:
-			self.load_stock_icon(id, icon, ( gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_SIZE_MENU, gtk.ICON_SIZE_BUTTON, ICON_SIZE_LABEL ))
+			self.create_stock_icon(id, icon, ( gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_SIZE_MENU, gtk.ICON_SIZE_BUTTON, gtk.ICON_SIZE_DIALOG, ICON_SIZE_LABEL, ICON_SIZE_HEADLINE ))
 
 
-	def load_icon(self, id, size):
-		"Loads an icon"
+	def get_iconsource(self, id, size, wildcard = False):
+		"Loads an icon as an iconsource"
 
-		if self.theme.has_icon(id):
-			pixbuf = self.theme.load_icon(id, size, 0)
+		width, height	= gtk.icon_size_lookup(size)
+		pixbuf		= self.get_pixbuf(id, width)
 
-		else:
+		if pixbuf == None:
+			return
+
+		# reject icons more than 4 pixels away from requested size if not wildcard
+		elif wildcard == False and not width - 4 <= pixbuf.get_property("width") <= width + 4:
+			return
+
+		elif wildcard == False and not height - 4 <= pixbuf.get_property("height") <= height + 4:
+			return
+
+		source = gtk.IconSource()
+		source.set_pixbuf(pixbuf)
+		source.set_size(size)
+		source.set_size_wildcarded(wildcard)
+
+		return source
+
+
+	def get_pixbuf(self, id, size):
+		"Loads an icon as a pixbuf"
+
+		if self.theme.has_icon(id) == False:
 			return None
 
-		return pixbuf
+		try:
+			return self.theme.load_icon(id, size, 0)
+
+		except gobject.GError:
+			return None
 
 
-	def load_stock_icon(self, id, icon, sizes):
-		"Registers a stock icon"
 
-		iconset = gtk.IconSet()
+##### ACTION HANDLING #####
 
-		if self.theme.has_icon(icon):
-			for size in dict.fromkeys(sizes).keys():
-				pixelsize = gtk.icon_size_lookup(size)[0]
+class Action(gtk.Action):
+	"UI Manager Action"
 
-				source = gtk.IconSource()
-				source.set_size(size)
-				source.set_size_wildcarded(False)
+	def __init__(self, name, label = None, tooltip = None, stock = "", important = False):
+		gtk.Action.__init__(self, name, label, tooltip, stock)
 
-				pixbuf = self.load_icon(icon, pixelsize)
+		if important == True:
+			self.set_property("is-important", True)
 
-				if pixbuf != None:
-					source.set_pixbuf(pixbuf)
 
-				iconset.add_source(source)
 
-		self.add(id, iconset)
+class ActionGroup(gtk.ActionGroup):
+	"UI Manager Actiongroup"
+
+	def add_action(self, action, accel = None):
+		"Adds an action to the actiongroup"
+
+		if accel == None:
+			gtk.ActionGroup.add_action(self, action)
+
+		else:
+			self.add_action_with_accel(action, accel)
+
+
+
+class ToggleAction(gtk.ToggleAction):
+	"A toggle action item"
+
+	def __init__(self, name, label, tooltip = None, stock = None):
+		gtk.ToggleAction.__init__(self, name, label, tooltip, stock)
 
 
 
@@ -1577,99 +1728,6 @@ class UIManager(gtk.UIManager):
 			widget.set_property("label", widget.get_property("label").replace("...", ""))
 
 
-	def add_actions_from_file(self, file):
-		"Sets up actions from an XML file"
-
-		data = io.file_read(file)
-		self.add_actions_from_string(data)
-
-
-	def add_actions_from_string(self, string):
-		"Sets up actions from an XML string"
-
-		try:
-			dom = xml.dom.minidom.parseString(string.strip())
-
-		except ExpatError:
-			raise DataError
-
-		if dom.documentElement.nodeName != "actions":
-			raise DataError
-
-
-		# load action groups
-		for groupnode in dom.documentElement.childNodes:
-
-			if groupnode.nodeType != groupnode.ELEMENT_NODE:
-				continue
-
-			if groupnode.nodeName != "actiongroup":
-				raise DataError
-
-			if not groupnode.attributes.has_key("name"):
-				raise DataError
-
-			actiongroup = gtk.ActionGroup(groupnode.attributes["name"].nodeValue)
-
-
-			# load actions
-			for actionnode in groupnode.childNodes:
-
-				if actionnode.nodeType != actionnode.ELEMENT_NODE:
-					continue
-
-				actiondata = {
-					"name"		: "",
-					"type"		: "normal",
-					"label"		: "",
-					"stock"		: "",
-					"accel"		: None,
-					"description"	: "",
-					"important"	: False
-				}
-
-				if actionnode.attributes.has_key("type"):
-					actiondata["type"] = actionnode.attributes["type"].nodeValue
-
-				if actionnode.attributes.has_key("important"):
-					actiondata["important"] = (actionnode.attributes["important"].nodeValue == "yes")
-
-				for node in actionnode.childNodes:
-
-					if node.nodeType != node.ELEMENT_NODE:
-						continue
-
-					elif actiondata.has_key(node.nodeName):
-						actiondata[node.nodeName] = util.dom_text(node)
-
-					else:
-						raise DataError
-
-				if actiondata["name"] == "":
-					raise DataError
-
-				if actiondata["type"] == "normal":
-					action = gtk.Action(
-						actiondata["name"], actiondata["label"],
-						actiondata["description"], actiondata["stock"]
-					)
-
-				elif actiondata["type"] == "toggle":
-					action = gtk.ToggleAction(
-						actiondata["name"], actiondata["label"],
-						actiondata["description"], actiondata["stock"]
-					)
-
-				else:
-					raise DataError
-
-				action.set_property("is-important", actiondata["important"])
-				actiongroup.add_action_with_accel(action, actiondata["accel"])
-
-
-			self.append_action_group(actiongroup)
-
-
 	def add_ui_from_file(self, file):
 		"Loads ui from a file"
 
@@ -1818,15 +1876,11 @@ class EntryView(VBox):
 
 	def __init__(self, cfg = None, clipboard = None):
 		VBox.__init__(self)
-		self.set_spacing(15)
-		self.set_border_width(10)
+		self.set_spacing(12)
+		self.set_border_width(12)
 
 		self.config		= cfg
 		self.clipboard		= clipboard
-
-		self.size_name		= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-		self.size_value		= gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-
 		self.entry		= None
 
 
@@ -1842,77 +1896,52 @@ class EntryView(VBox):
 	def display_entry(self, e):
 		"Displays info about an entry"
 
-		if e is None:
-			if self.entry is not None:
-				self.clear()
-
-			return
-
 		self.clear()
 		self.entry = e
 
+		if self.entry == None:
+			return
+
 		# set up metadata display
 		metabox = VBox()
 		self.pack_start(metabox)
 
-		metabox.pack_start(ImageLabel(
+		label = ImageLabel(
 			"<span size=\"large\" weight=\"bold\">%s</span>" % util.escape_markup(e.name),
 			e.icon, ICON_SIZE_DATAVIEW
-		))
+		)
+		metabox.pack_start(Alignment(label, 0.5, 0.5, 0, 0))
 
-		metabox.pack_start(Label("<span weight=\"bold\">%s</span>%s" % ( e.typename + (e.description != "" and "; " or ""), util.escape_markup(e.description) ), gtk.JUSTIFY_CENTER))
+		label = Label("<span weight=\"bold\">%s</span>%s" % ( e.typename + (e.description != "" and ": " or ""), util.escape_markup(e.description) ), gtk.JUSTIFY_CENTER)
+		metabox.pack_start(label)
 
 		# set up field list
 		fields = [ field for field in e.fields if field.value != "" ]
 
 		if len(fields) > 0:
-			fieldlist = VBox()
-			fieldlist.set_spacing(2)
-			self.pack_start(fieldlist)
+			table = Table(2, len(fields))
+			self.pack_start(table)
 
-			for field in fields:
-				row = HBox()
-				fieldlist.pack_start(row, False, False)
-
-				label = Label("<span weight=\"bold\">%s:</span>" % util.escape_markup(field.name), gtk.JUSTIFY_RIGHT)
-				self.size_name.add_widget(label)
-				row.pack_start(label, False, False)
+			for rowindex, field in zip(range(len(fields)), fields):
+				label = Label("<span weight=\"bold\">%s:</span>" % util.escape_markup(field.name))
+				table.attach(label, 0, rowindex)
 
 				widget = generate_field_display_widget(field, self.config, self.clipboard)
-				self.size_value.add_widget(widget)
-				row.pack_start(widget, False, False)
+				table.attach(widget, 1, rowindex)
 
 		# display updatetime
 		if type(e) != entry.FolderEntry:
-			self.pack_start(Label("Updated %s ago;\n%s" % ( util.time_period_rough(e.updated, time.time()), time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(e.updated)) ), gtk.JUSTIFY_CENTER))
+			label = Label((_('Updated %s ago') + "\n%s") % ( util.time_period_rough(e.updated, time.time()), time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(e.updated)) ), gtk.JUSTIFY_CENTER)
+			self.pack_start(label)
 
 		self.show_all()
 
 
-	def display_info(self):
-		"Displays info about the application"
-
-		self.clear()
-
-		self.pack_start(ImageLabel(
-			"""<span size="x-large" weight="bold">%s %s</span>""" % ( config.APPNAME, config.VERSION ),
-			STOCK_REVELATION, ICON_SIZE_LOGO
-		))
-
-		self.pack_start(Label("A password manager for GNOME 2"))
-	
-		gpl = "\nThis program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details."
-		label = Label("<span size=\"x-small\">" + gpl + "</span>", gtk.JUSTIFY_LEFT)
-		label.set_size_request(250, -1)
-		self.pack_start(label)
-
-
 	def pack_start(self, widget):
 		"Adds a widget to the data view"
 
-		alignment = gtk.Alignment(0.5, 0.5, 0, 0)
-		alignment.add(widget)
-		VBox.pack_start(self, alignment)
+		alignment = Alignment(widget, 0.5, 0.5, 0, 0)
+		VBox.pack_start(self, alignment, False, False)
 
 
 
@@ -1922,19 +1951,19 @@ class Searchbar(Toolbar):
 	def __init__(self):
 		Toolbar.__init__(self)
 
-		self.label		= Label("  Find: ")
+		self.label		= Label("  " + _('  Find:') + " ")
 		self.entry		= Entry()
 		self.dropdown		= EntryDropDown()
-		self.dropdown.insert_item(0, "Any type", "gnome-stock-about")
+		self.dropdown.insert_item(0, _('Any type'), "gnome-stock-about")
 		self.button_next	= Button(STOCK_NEXT)
 		self.button_prev	= Button(STOCK_PREVIOUS)
 
 		self.append_widget(self.label)
-		self.append_widget(self.entry, "Text to search for")
-		self.append_widget(EventBox(self.dropdown), "The type of account to search for")
+		self.append_widget(self.entry, _('Text to search for'))
+		self.append_widget(EventBox(self.dropdown), _('The type of account to search for'))
 		self.append_space()
-		self.append_widget(self.button_next, "Find the next match")
-		self.append_widget(self.button_prev, "Find the previous match")
+		self.append_widget(self.button_next, _('Find the next match'))
+		self.append_widget(self.button_prev, _('Find the previous match'))
 
 		self.connect("show", self.__cb_show)
 
